const Promise = require('bluebird');
const CFError = require('cf-errors');
const { promisify } = require('util');
const requestRetry = require('request-retry');

const retriableErrors = [
    'ECONNRESET',
    'ENOTFOUND',
    'ESOCKETTIMEDOUT',
    'ETIMEDOUT',
    'ECONNREFUSED',
    'EHOSTUNREACH',
    'EPIPE',
    'EAI_AGAIN',
    'EBUSY',
];
const retriableStatusCodes = [
    503,
    412, // https://developer.hashicorp.com/vault/docs/enterprise/consistency
];

const rr = requestRetry({
    retries: 10,
    detectors: [
        (err) => {
            return err && (
                retriableErrors.includes(err.code)
                || retriableStatusCodes.includes(err.statusCode)
            );
        },
        (err, response) => {
            return response && (
                retriableErrors.includes(response.code)
                || retriableStatusCodes.includes(response.statusCode)
            );
        }
    ]
});
const rp = promisify(rr.request.bind(rr));

async function to(fn) {
    try {
        let res;
        if (typeof fn === 'function') {
            res = await fn();
        } else {
            res = await fn;
        }
        return [null, res];
    } catch (err) {
        return [err, null];
    }
}

/**
 * extract the parts of a variable reference
 * @param {string} variable variable
 * @returns {[string, string, string]} an array with the namespace, resource-name, and key
 */
function _splitToParts(variable = '') {
    const res = variable.split('.');
    const namespace = res.shift();
    const name = res.shift();
    return [namespace, name, (res || []).join('.')];
}

/**
 * builds an error object
 * @param {{lvl: string, message: string, default: string}} opt options
 * @param {Error} error error
 * @param {string} suggestion suggestion
 * @returns {CFError} error
 */
function _buildError(opt, error, suggestion) {
    const data = {
        message: opt.message,
        cause: error,
        lvl: opt.lvl,
    };

    if (opt.default) {
        data.default = opt.default;
    }
    if (error) {
        data.message = `${opt.message}. ${error.message}`;
    }
    if (suggestion) {
        data.suggestion = suggestion;
    }
    return new CFError(data);
}

/**
 * perform an operation with a set timeout
 * @param {function(...*):Promise<*>} fn
 * @param {number} timeout
 * @returns {function(...*):Promise<*>} function with timeout
 */
function _withTimeout(fn, timeout) {
    return function (...args) {
        return Promise.resolve()
            .then(() => fn(...args))
            .timeout(timeout);
    };
}

/**
 * checks if the given response contains a json body
 * @param {Object} res http response
 * @returns {boolean}
 */
const isJsonResponse = (res) => {
    const contentType = res.headers.get('content-type');
    return contentType && contentType.indexOf('application/json') !== -1;
};

module.exports = {
    to,
    split: _splitToParts,
    buildError: _buildError,
    withTimeout: _withTimeout,
    isJsonResponse,
    requestPromise: rp,
};
